﻿using OpcLabs.BaseLib.ComponentModel;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.DataAccess;

using System.Reflection;

namespace ConsoleApp1
{
    internal class Program
    {
        static void Main(string[] args)
        {
            //This application is displaying an UnobservedTaskException, which seems to stem from the QuickOPC toolkit
            
            AppDomain.CurrentDomain.UnhandledException += CurrentDomain_UnhandledException;
            TaskScheduler.UnobservedTaskException += TaskScheduler_UnobservedTaskException;

            //All of the below code blocks perform a different QuickOPC procedure. Uncomment one at a time to see the result.

            //Create a isolated shared instance of an EasyUAClient object, but perform no further action -> Error occurs 
            //EasyUAClient.SharedInstance.Isolated = true;

            //Create a shared instance of an EasyUAClient object and perform a read on an OPC UA node -> Error occurs
            //object value;
            //value = EasyUAClient.SharedInstance.ReadValue("opc.tcp://localhost:49380", "ns=2;s=Channel3.Device1.Tag1");
            //Console.WriteLine($"value: {value}");

            //Create an EasyUAClient object and perform a read on an OPC UA node -> Error occurs 
            //EasyUAClient client = new EasyUAClient();
            //object value;
            //value = client.ReadValue("opc.tcp://localhost:49380", "ns=2;s=Channel3.Device1.Tag1");
            //Console.WriteLine($"value: {value}");

            //Create an EasyDAClient object and perform a read on an OPC DA item -> No error
            //EasyDAClient DAClient = new EasyDAClient();
            //object value;
            //value = DAClient.ReadItemValue("SWToolbox.TOPServer.V6", "Channel3.Device1.Tag1");
            //Console.WriteLine($"value: {value}");

            Console.WriteLine("");
            _ = Task.Run(async () =>
            {
                for (int i = 0; i < 25; i++)
                {
                    Console.Write(i);
                    await Task.Delay(100);
                }
                Console.WriteLine("");
                GC.Collect(2, GCCollectionMode.Forced);
            });

            Console.WriteLine("Press Any Key to Exit");
            GC.Collect(2, GCCollectionMode.Forced);
            Console.ReadKey();

            Console.WriteLine("Exiting");
        }

        private static void TaskScheduler_UnobservedTaskException(object sender, UnobservedTaskExceptionEventArgs e)
        {
            Console.WriteLine("");
            Console.WriteLine("Unobserved Task Exception: {0}", e.Exception.InnerException.Message);
        }

        private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            Console.WriteLine("Unhandled Exception");
        }
    }
}